#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import configparser
from gettext import gettext as _

from gi import require_version
require_version("Gtk", "3.0")

from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GLib

#from .configuration import *
#from .utils import *

class PreferencesWindow(Gtk.Window):

    def __init__(self, main_window = None):
        Gtk.Window.__init__(self)

        self.main_window = main_window
        self.connect("delete-event", self._quit)
        self.connect("key-press-event", self._keypress)

        """
        Build Settings
        """
        self.set_title(_("Preferences"))
        self.set_default_size(480, 400)
        self.set_modal(True)
        self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
        self.set_transient_for(self.main_window)
        self.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)

        """
        Build HeaderBar
        """
        self.headerbar = Gtk.HeaderBar()
        self.headerbar.set_title(_("Preferences"))
        self.headerbar.set_show_close_button(True)
        self.set_titlebar(self.headerbar)

        """
        Preferences stores
        """
        self.pages = {}
        self.listboxes = {}
        self.preferences = None
        self.listbox = None
        self.notebook = None

        """
        Build Content
        """
        self.build()

    def _keypress(self, widget, event):
        if event.keyval == Gdk.KEY_Escape:
            #self.destroy()
            Gtk.main_quit()

    def _quit(self, widget, event):
        #self.destroy()
        Gtk.main_quit()

    def build(self):
        """
        Build Layout
        """
        self.layout = Gtk.Grid()
        self.layout.set_orientation(Gtk.Orientation.VERTICAL)
        self.layout.set_border_width(0)
        self.layout.set_column_homogeneous(True)
        self.add(self.layout)

        """
        Build Content
        """
        self._build_preferences()

        """
        Build Search
        """
        self._build_search()

    def _build_preferences(self):
        # Created scrolled region
        self.scrolled_window = Gtk.ScrolledWindow()
        self.scrolled_window.set_vexpand(True)
        self.layout.attach(self.scrolled_window, 0, 1, 1, 1)

        # Create flowbox
        self.flowbox = Gtk.FlowBox()
        self.flowbox.set_valign(Gtk.Align.START)
        self.flowbox.set_max_children_per_line(1)
        self.flowbox.set_selection_mode(Gtk.SelectionMode.NONE)
        self.flowbox.set_margin_top(5)
        self.scrolled_window.add(self.flowbox)

    def set(self):
        pass

    def set_row(self, widget, tab = None):


        self.flowbox.add(widget)











    def add_preference(self, title, type, callback, data = None, page = None):
        ## Row
        ### Create list row
        row = Gtk.FlowBoxChild()
        row.set_can_focus(False)
        row.set_margin_left(10)
        row.set_margin_right(10)
        row.set_margin_top(5)
        row.set_margin_bottom(5)

        ### Create list row content box
        box = Gtk.Box()
        box.set_orientation(Gtk.Orientation.HORIZONTAL)
        box.set_homogeneous(True)

        #### Create list row content box label
        label = Gtk.Label()
        label.set_text(_(title))
        label.set_halign(Gtk.Align.START)
        label.set_valign(Gtk.Align.CENTER)
        box.pack_start(label, True, True, 0)

        #### Create list row content box switch
        switch = Gtk.Switch()
        switch.set_halign(Gtk.Align.END)
        switch.set_valign(Gtk.Align.CENTER)
        switch.set_active(True)
        switch.connect("notify::active", callback)
        row.set_focus_child(switch)
        box.pack_start(switch, False, True, 0)

        ### Row content add to listbox row
        row.add(box)
        self.flowbox.add(row)

    def _build_search(self):
        self.search_icon = Gtk.Image()
        self.search_icon.set_from_icon_name("edit-find-symbolic", Gtk.IconSize.BUTTON)
        self.search_button = Gtk.ToggleButton()
        self.search_button.add(self.search_icon)
        self.search_button.connect("toggled", self._on_transition)
        self.headerbar.pack_start(self.search_button)

        self.search_revealer = Gtk.Revealer()
        self.search_entry = Gtk.SearchEntry()
        self.search_entry.set_placeholder_text(_("Search Preferences"))
        self.search_entry.set_margin_left(10)
        self.search_entry.set_margin_right(10)
        self.search_entry.set_margin_top(10)
        self.search_entry.set_margin_bottom(5)
        self.search_entry.connect("search-changed", self._on_search)
        self.search_revealer.add(self.search_entry)

        self.layout.attach(self.search_revealer, 0, 0, 1, 1)

        self.flowbox.set_filter_func(self._on_search_filter, None)

    def _on_search(self, entry):
        self.flowbox.invalidate_filter()

    def _on_search_filter(self, lista, user_data):
        text = self.search_entry.get_text()
        if not text:
            return lista
        """
        lista        (Gtk.ListBox) -> .get_child
        get the rows (Gtk.ListBoxRow) -> .git_children
        get the label Gtk.Box -> [0] (a.k.a label)
        """
        lbl = lista.get_child().get_children()[0]
        if text.lower() in lbl.get_text().lower():
            return lista

    def _on_transition(self, btn):
        if self.search_revealer.get_reveal_child():
            self.search_revealer.set_reveal_child(False)
            self.search_entry.set_text("")
            btn.grab_focus()
        else:
            self.search_revealer.set_reveal_child(True)
            self.search_entry.grab_focus()

    def _on_key_press(self, widget, event):
        keyname = Gdk.keyval_name(event.keyval)
        if keyname == 'Escape':
            self.search_button.set_active(False)
        if event.state and Gdk.ModifierType.CONTROL_MASK:
            if keyname == 'f':
                self.search_button.set_active(True)


class Preferences(object):

    def __init__(self):
        pass

    def file(self, file = None):
        pass

    def add(self):
        pass




def on_switch_activated(switch, gparam):
    if switch.get_active():
        state = "on"
    else:
        state = "off"

    print("Switch was turned", state)






preferencesWindow = PreferencesWindow()

widget1 = Gtk.Label()
widget1.set_text("Test")
widget2 = Gtk.Label()
widget2.set_text("Test 2")
widget3 = Gtk.Label()
widget3.set_text("Test 3")

preferencesWindow.set_row(widget1, tab = "General")
preferencesWindow.set_row(widget2, tab = "General")
preferencesWindow.set_row(widget3, tab = "Video")

preferencesWindow.show_all()
Gtk.main()








"""
preferencesWindow = PreferencesWindow(main_window=self.window)

preferencesWindow.set(
    section = "",
    label = "",
    description = "",
    type = "",
    value = "",
    default = "",
    callback = "",
    options = "",
    tab = ""
)

widget = Gtk.Label()
widget.set_text("Test")

preferencesWindow.set_row(widget, tab = "General")


preferencesWindow.show_all()
Gtk.main()
"""




"""
window = PreferencesWindow()

window.add_preference("Title of option", "switch", on_switch_activated, None)
window.add_preference("Title of option 2", "switch", on_switch_activated, None)
window.add_preference("Title of option 3", "switch", on_switch_activated, None)
window.add_preference("Title of option 4", "switch", on_switch_activated, None)
window.add_preference("Title of option 5", "switch", on_switch_activated, None)
window.add_preference("Title of option 6", "switch", on_switch_activated, None)

window.add_preference("Title of option", "switch", on_switch_activated, None, "General")
window.add_preference("Title of option 2", "switch", on_switch_activated, None, "General")
window.add_preference("Title of option 3", "switch", on_switch_activated, None, "General")


window.show_all()
Gtk.main()
"""


"""
if is_linux():
    #config_file = os.path.expanduser("~/.config/{0}/{0}.ini".format(APP_COMMAND))
    config_file = "config.ini"

preferences = Preferences()
preferences.file(config_file)

preferences.set(section, key, value, default, {
    tab: "",
    label: "",
    type: "",
    callback: "",
    description: "",
    options: ""
})

pref = preferences.get()
preferences.pref[name].value
print (pref.value)
print (pref.default)

preferences.window(main_window=self.window)


preferencesWindow = PreferencesWindow(main_window=self.window)

preferencesWindow.set(
    section = "",
    label = "",
    description = "",
    type = "",
    value = "",
    default = "",
    callback = "",
    options = "",
    tab = ""
)

preferencesWindow.show_all()















# Inside preferences.window()
self.window = PreferencesWindow(main)

for pref in self.preferencesWindow:
    self.window.set(label, type, callback, options, section, description)
"""
