#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from setuptools import setup
from subprocess import call

def post_install():
    # Does this need to run? I don't known!
    try:
        call(["glib-compile-schemas", "/usr/share/glib-2.0/schemas/"])
    except OSError as e:
        if e.errno == os.errno.ENOENT:
            print("The command \"glib-compile-schemas\" does not exists, please install it then run:\n$ glib-compile-schemas /usr/share/glib-2.0/schemas/")
        else:
            print("glib-compile-schemas failed for soom unknown reason, try running it in your shell:\n$ glib-compile-schemas /usr/share/glib-2.0/schemas/")

# OS specific settings
if "nt" == os.name:
    data_files = []
else:
    data_files = [
        ("", ["LICENSE", "README"]),
        # Default install /usr/local/...
        ("/usr/share/applications", ["data/org.gas.importer.desktop"]),
        ("/usr/share/appdata/", ["data/org.gas.importer.appdata.xml"]),
        ("/usr/share/glib-2.0/schemas/", ["data/org.gas.importer.gschema.xml"]),
        
        ("/usr/share/icons/hicolor/scalable/apps", ["data/icons/scalable/org.gas.importer.svg"]),
        #("share/icons/hicolor/16x16/apps", ["data/icons/16x16/org.gas.importer.png"]),
        #("share/icons/hicolor/48x48/apps", ["data/icons/48x48/org.gas.importer.png"]),
        #("share/icons/hicolor/64x64/apps", ["data/icons/64x64/org.gas.importer.png"]),
        #("share/icons/hicolor/128x128/apps", ["data/icons/128x128/org.gas.importer.png"]),
        ("/usr/share/icons/hicolor/256x256/apps", ["data/icons/256x256/org.gas.importer.png"]),
        #("share/icons/hicolor/512x512/apps", ["data/icons/512x512/org.gas.importer.png"])
    ]

# Read README as long description if we can
if os.path.isfile("README"):
    with open("README") as readme:
        long_description = readme.read()
else:
    long_description = ""

# Setup
setup(
    name = "gas-importer",
    version = "0.1.0",
    description = "Media importer for Go Amateur Studio.",
    long_description = long_description,
    author = "Julian Richen",
    author_email = "julian@richen.io",
    url = "https://github.com/gas/importer",
    license = "GPLv3",
    packages = [
        "gasimporter"
    ],
    entry_points = {
        "console_scripts": [
            "gas-importer=gasimporter.application:main"
        ],
    },
    data_files = data_files,
    cmdclass = {
        #'develop': post_install_dev,
        'install': post_install,
    }
)